#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

void exampleIo(RpcClientPtr impl)
{
    auto robot_name = impl->getRobotNames().front();
    auto robot = impl->getRobotInterface(robot_name);

    int default_ai_num = robot->getIoControl()->getStandardAnalogInputNum();
    int default_ao_num = robot->getIoControl()->getStandardAnalogOutputNum();
    int default_do_num = robot->getIoControl()->getStandardDigitalOutputNum();
    int default_di_num = robot->getIoControl()->getStandardDigitalInputNum();

    std::cout << "StandardDigitalInputNum: "
              << robot->getIoControl()->getStandardDigitalInputNum()
              << std::endl;
    std::cout << "StandardDigitalOutputNum: "
              << robot->getIoControl()->getStandardDigitalOutputNum()
              << std::endl;
    std::cout << "ToolDigitalInputNum: "
              << robot->getIoControl()->getToolDigitalInputNum() << std::endl;
    std::cout << "ToolDigitalOutputNum: "
              << robot->getIoControl()->getToolDigitalOutputNum() << std::endl;
    std::cout << "ConfigurableDigitalInputNum: "
              << robot->getIoControl()->getConfigurableDigitalInputNum()
              << std::endl;
    std::cout << "ConfigurableDigitalOutputNum: "
              << robot->getIoControl()->getConfigurableDigitalOutputNum()
              << std::endl;

    int ret = 0;
    ret = robot->getIoControl()->setStandardDigitalInputAction(
        0x00000001, StandardInputAction::Default);
    ret = robot->getIoControl()->setStandardDigitalOutputRunstate(
        0x00000001, StandardOutputRunState::NONE);
    /*ret = impl->getIoControl()->setConfigurableInputAction(
        0x00000100, StandardInputAction::Freedrive);
    ret = impl->getIoControl()->setConfigurableOutputRunstate(
        0x00000100, StandardOutputRunState::STOP_HIGH);*/
    ret = robot->getIoControl()->setStandardAnalogOutputRunstate(
        0x00000010, StandardOutputRunState::RUNNING);
    ret = robot->getIoControl()->setStandardAnalogInputDomain(0, 12);
    ret = robot->getIoControl()->setStandardAnalogOutputDomain(0, 12);

    // configure io
    // [已测试] Digital IO [I系列IO控制响应频率:20hz(max)]
    for (int i = 0; i < default_do_num; i++) {
        ret = robot->getIoControl()->setStandardDigitalOutput(i, false);
        std::this_thread::sleep_for(std::chrono::seconds(1));
        auto ret_b = robot->getIoControl()->getStandardDigitalOutput(i);
        std::cout << "Digital output:" << ret_b << std::endl;
    }

    for (int i = 0; i < default_di_num; i++) {
        auto ret_b = robot->getIoControl()->getStandardDigitalInput(i);
        std::cout << "Digital input:" << ret_b << std::endl;
    }

    // Analog IO (四路模拟电压输入，两路模拟电压电流输出)
    ret = robot->getIoControl()->setStandardAnalogOutput(0x00000000,
                                                         5); // 0,1 Voltage V
    std::this_thread::sleep_for(std::chrono::seconds(1));
    ret = robot->getIoControl()->setStandardAnalogOutput(1, 10);
    std::this_thread::sleep_for(std::chrono::seconds(1));
    ret = robot->getIoControl()->setStandardAnalogOutput(2,
                                                         10); // 2,3 Current ma
    std::this_thread::sleep_for(std::chrono::seconds(1));
    ret = robot->getIoControl()->setStandardAnalogOutput(3, 10);
    std::this_thread::sleep_for(std::chrono::seconds(1));
    double ret_d = 0.0;
    for (int i = 0; i < default_ai_num; i++) {
        ret_d = robot->getIoControl()->getStandardAnalogInput(i);
        std::this_thread::sleep_for(std::chrono::seconds(1));
        std::cout << "Analog input Voltage:" << ret_d << std::endl;
    }
}

#define ip_local "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(ip_local, 30004);
    rpc_cli->login("aubo", "123456");

    exampleIo(rpc_cli);
}
