#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

void exampleRpyToQuat(RpcClientPtr impl)
{
    std::cout << std::endl << "欧拉角 -> 四元数 " << std::endl;

    std::vector<double> rpy;
    rpy.push_back(0.611);
    rpy.push_back(0.785);
    rpy.push_back(0.960);
    std::cout << "rx:" << rpy.at(0) << std::endl;
    std::cout << "ry:" << rpy.at(1) << std::endl;
    std::cout << "rz:" << rpy.at(2) << std::endl;

    auto robot_name = impl->getRobotNames().front();
    std::vector<double> quat = impl->getRobotInterface(robot_name)
                                   ->getRobotAlgorithm()
                                   ->rpyToQuaternion(rpy);

    if (4 == quat.size()) {
        std::cout << "w:" << quat.at(0) << std::endl;
        std::cout << "x:" << quat.at(1) << std::endl;
        std::cout << "y:" << quat.at(2) << std::endl;
        std::cout << "z:" << quat.at(3) << std::endl;
    }
}

void exampleQuatToRpy(RpcClientPtr impl)
{
    std::cout << std::endl << "四元数 -> 欧拉角" << std::endl;

    std::vector<double> quat;
    quat.push_back(0.789);
    quat.push_back(0.153);
    quat.push_back(0.001);
    quat.push_back(0.595);
    std::cout << "w:" << quat.at(0) << std::endl;
    std::cout << "x:" << quat.at(1) << std::endl;
    std::cout << "y:" << quat.at(2) << std::endl;
    std::cout << "z:" << quat.at(3) << std::endl;

    auto robot_name = impl->getRobotNames().front();
    std::vector<double> rpy = impl->getRobotInterface(robot_name)
                                  ->getRobotAlgorithm()
                                  ->quaternionToRpy(quat);

    if (3 == rpy.size()) {
        std::cout << "rx:" << rpy.at(0) << std::endl;
        std::cout << "ry:" << rpy.at(1) << std::endl;
        std::cout << "rz:" << rpy.at(2) << std::endl;
    }
}

std::tuple<std::vector<double>, int> res;

void exampleInverseK(RpcClientPtr impl)
{
    std::cout << std::endl << "逆解" << std::endl;
    auto robot_name = impl->getRobotNames().front();
    auto cur_joint = impl->getRobotInterface(robot_name)
                         ->getRobotState()
                         ->getJointPositions();

    //机械臂目标点
    std::vector<double> Waypoint_1_p = { 0.549273, -0.12094,   0.464501,
                                         3.13802,  0.00122674, 1.57095 };
    // 机械臂当前位置作为参考点
    std::vector<double> Waypoint_0_q = { 0.00123059, -0.26063, 1.7441,
                                         0.437503,   1.56957,  0.00107586 };

    res = impl->getRobotInterface(robot_name)
              ->getRobotAlgorithm()
              ->inverseKinematics(Waypoint_0_q, Waypoint_1_p);

    std::cout << "逆解位置" << std::endl;
    for (int i = 0; i < 6; i++) {
        std::cout << Waypoint_1_p.at(i) << ",";
    }
    std::cout << std::endl << "逆解结果:" << std::get<1>(res) << std::endl;
    for (int i = 0; i < 6; i++) {
        std::cout << std::get<0>(res).at(i) << ",";
    }
}

void exampleForwardK(RpcClientPtr impl)
{
    std::cout << std::endl << "正解" << std::endl;

    auto robot_name = impl->getRobotNames().front();
    auto res_pose = impl->getRobotInterface(robot_name)
                        ->getRobotAlgorithm()
                        ->forwardKinematics(std::get<0>(res));

    std::cout << "正解关节" << std::endl;
    for (int i = 0; i < 6; i++) {
        std::cout << std::get<0>(res).at(i) << ",";
    }
    std::cout << std::endl << "正解结果:" << std::get<1>(res_pose) << std::endl;
    for (int i = 0; i < 6; i++) {
        std::cout << std::get<0>(res_pose).at(i) << ",";
    }
}

#define ip_local "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(ip_local, 30004);
    rpc_cli->login("aubo", "123456");

    exampleRpyToQuat(rpc_cli);
    exampleQuatToRpy(rpc_cli);
    exampleInverseK(rpc_cli);
    exampleForwardK(rpc_cli);
}
