#include <math.h>
#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

double distance(const std::vector<double> &a, const std::vector<double> &b)
{
    double res = 0.;
    if (a.size() != b.size()) {
        return -1;
    }

    for (int i = 0; i < (int)a.size(); i++) {
        res += (a[i] - b[i]) * (a[i] - b[i]);
    }
    return sqrt(res);
}

void waitArrivel(RobotInterfacePtr impl, std::vector<double> target)
{
    while (1) {
        std::vector cur = impl->getRobotState()->getJointPositions();
        double dis = distance(cur, target);
        if (dis < 0.0001) {
            break;
        }
        printf("cur:%f,%f,%f,%f,%f,%f\n", cur.at(0), cur.at(1), cur.at(2),
               cur.at(3), cur.at(4), cur.at(5));
        printf("target:%f,%f,%f,%f,%f,%f\n", target.at(0), target.at(1),
               target.at(2), target.at(3), target.at(4), target.at(5));
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }
}

void exampleMovej(RpcClientPtr cli)
{
    std::vector<double> q1 = { -2.05177,  -0.400292, 1.19625,
                               0.0285152, 1.57033,   -2.28774 };

    std::vector<double> q2 = { -0.236948,   -0.465633, 1.1002,
                               -0.00600456, 1.56824,   -0.472864 };

    std::vector<double> q3 = { 1.38167,  -0.0115091, 1.66958,
                               0.107726, 1.572,      1.14576 };

    // move q1
    printf("goto p1\n");

    auto robot_name = cli->getRobotNames().front();

    cli->getRuntimeMachine()->start();
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0);
    waitArrivel(cli->getRobotInterface(robot_name), q1);

    // move q2
    printf("goto p2\n");
    cli->getRuntimeMachine()->start();
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q2, 90 * (M_PI / 180), 180 * (M_PI / 180), 0, 0);

    waitArrivel(cli->getRobotInterface(robot_name), q2);

    // move q3
    printf("goto p3\n");
    cli->getRuntimeMachine()->start();
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q3, 90 * (M_PI / 180), 180 * (M_PI / 180), 0, 0);

    waitArrivel(cli->getRobotInterface(robot_name), q3);
}

#define ip_local "127.0.0.1"

int main(int argc, char **argv)
{
    auto cli = std::make_shared<RpcClient>();
    cli->connect(ip_local, 30004);
    cli->login("aubo", "123456");

    exampleMovej(cli);
}
