#include <math.h>
#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

double distance(const std::vector<double> &a, const std::vector<double> &b)
{
    double res = 0.;
    if (a.size() != b.size()) {
        return -1;
    }

    for (int i = 0; i < (int)a.size(); i++) {
        res += (a[i] - b[i]) * (a[i] - b[i]);
    }
    return sqrt(res);
}

void waitArrivel(RobotInterfacePtr impl, std::vector<double> target)
{
    while (1) {
        std::vector cur = impl->getRobotState()->getJointPositions();
        double dis = distance(cur, target);
        if (dis < 0.0001) {
            break;
        }
        printf("cur:%f,%f,%f,%f,%f,%f\n", cur.at(0), cur.at(1), cur.at(2),
               cur.at(3), cur.at(4), cur.at(5));
        printf("target:%f,%f,%f,%f,%f,%f\n", target.at(0), target.at(1),
               target.at(2), target.at(3), target.at(4), target.at(5));
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }
}

void exampleMovel(RpcClientPtr impl)
{
    std::vector<double> q1 = { -2.40194, 0.103747, 1.7804,
                               0.108636, 1.57129,  -2.6379 };

    std::vector<double> p1 = { 0.764244, -0.0291056, 0.439062,
                               3.05162,  0.032431,   1.80416 };

    std::vector<double> Waypoint_0_p = { -0.155944, -0.727344, 0.439066,
                                         3.05165,   0.0324355, 1.80417 };
    std::vector<double> Waypoint_0_q = { -1.63013,  -0.727936, 0.558376,
                                         -0.279454, 1.66626,   -1.86472 };
    std::vector<double> Waypoint_1_p = { -0.581143, -0.357548, 0.439066,
                                         3.05165,   0.0324355, 1.80417 };
    std::vector<double> Waypoint_1_q = { -2.42089,  -0.529301, 0.867566,
                                         -0.238376, 1.64145,   -2.658 };
    std::vector<double> Waypoint_2_p = { 0.503502, -0.420646, 0.439066,
                                         3.05165,  0.0324355, 1.80417 };
    std::vector<double> Waypoint_2_q = { -0.515177, -0.402876, 1.09792,
                                         0.0179699, 1.60827,   -0.748361 };

    auto robot_name = impl->getRobotNames().front();

    impl->getRuntimeMachine()->start();
    impl->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0);
    waitArrivel(impl->getRobotInterface(robot_name), q1);

    int ret = 0;
    int cnt = 0;
    std::vector<double> cur;
    std::vector<double> inverse_value;

    for (;;) {
        // move p1
        printf("goto p1\n");
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_0_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);

        while (1) {
            cur.clear();
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_0_p);
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            if (cnt++ > 10) {
                exit(-1);
            }

            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        waitArrivel(impl->getRobotInterface(robot_name), inverse_value);
        inverse_value.clear();

        // move p2
        printf("goto p2\n");
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_1_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);
        while (1) {
            cur.clear();
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_1_p);
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            if (cnt++ > 10) {
                exit(-1);
            }
            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        waitArrivel(impl->getRobotInterface(robot_name), inverse_value);
        inverse_value.clear();

        // move p3
        printf("goto p3\n");
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_2_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);
        while (1) {
            cur.clear();
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_2_p);
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            if (cnt++ > 10) {
                exit(-1);
            }
            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        waitArrivel(impl->getRobotInterface(robot_name), inverse_value);
        inverse_value.clear();
    }
}

#define ip_local "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(ip_local, 30004);
    rpc_cli->login("aubo", "123456");

    exampleMovel(rpc_cli);
}
