#include "aubo_sdk/rpc.h"

#define ip_local "127.0.0.1"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

void printfTitle(std::string str)
{
    std::cout << std::endl << std::endl << str << std::endl;
}

void exampleRegisterInput(RpcClientPtr impl)
{
    printfTitle("->Register_Bool_Input_0_63");
    for (int i = 0; i < 63; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setBoolInput(i, true);
        } else {
            impl->getRegisterControl()->setBoolInput(i, false);
        }
        std::cout << (int)impl->getRegisterControl()->getBoolInput(i) << ",";
    }

    printfTitle("->Register_Bool_Input_64_127");
    for (int i = 64; i < 128; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setBoolInput(i, true);
        } else {
            impl->getRegisterControl()->setBoolInput(i, false);
        }
        std::cout << (int)impl->getRegisterControl()->getBoolInput(i) << ",";
    }

    printfTitle("->Register_Int_Input");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setInt32Input(i, i);
        } else {
            impl->getRegisterControl()->setInt32Input(i, i);
        }
        std::cout << impl->getRegisterControl()->getInt32Input(i) << ",";
    }

    printfTitle("->Register_Float_Input");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setFloatInput(i, 0.1);
        } else {
            impl->getRegisterControl()->setFloatInput(i, 0.2);
        }
        std::cout << impl->getRegisterControl()->getFloatInput(i) << ",";
    }

    printfTitle("->Register_Double_Input");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setDoubleInput(i, 0.3);
        } else {
            impl->getRegisterControl()->setDoubleInput(i, 0.4);
        }
        std::cout << impl->getRegisterControl()->getDoubleInput(i) << ",";
    }
}

void exampleRegisterOutput(RpcClientPtr impl)
{
    printfTitle("->Register_Bool_Output_0_63");
    for (int i = 0; i < 63; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setBoolOutput(i, true);
        } else {
            impl->getRegisterControl()->setBoolOutput(i, false);
        }
        std::cout << (int)impl->getRegisterControl()->getBoolOutput(i) << ",";
    }

    printfTitle("->Register_Bool_Output_64_127");
    for (int i = 64; i < 127; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setBoolOutput(i, true);
        } else {
            impl->getRegisterControl()->setBoolOutput(i, false);
        }
        std::cout << (int)impl->getRegisterControl()->getBoolOutput(i) << ",";
    }

    printfTitle("->Register_Int_Output");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setInt32Output(i, i);
        } else {
            impl->getRegisterControl()->setInt32Output(i, i);
        }
        std::cout << impl->getRegisterControl()->getInt32Output(i) << ",";
    }

    printfTitle("->Register_Float_Output");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setFloatOutput(i, 0.1);
        } else {
            impl->getRegisterControl()->setFloatOutput(i, 0.2);
        }
        std::cout << impl->getRegisterControl()->getFloatOutput(i) << ",";
    }

    printfTitle("->Register_Double_Output");
    for (int i = 0; i < 64; i++) {
        if (0 == i % 2) {
            impl->getRegisterControl()->setDoubleOutput(i, 0.3);
        } else {
            impl->getRegisterControl()->setDoubleOutput(i, 0.4);
        }
        std::cout << impl->getRegisterControl()->getDoubleOutput(i) << ",";
    }
}

void exampleRegisterCustom(RpcClientPtr impl)
{
    impl->getRegisterControl()->setBoolReg("custom1", true);
    bool v_b = false;
    v_b = impl->getRegisterControl()->getBoolReg("custom1", v_b);
    std::cout << std::endl
              << std::endl
              << "->Register_Bool_Custom:" << v_b << std::endl;

    impl->getRegisterControl()->setInt32Reg("custom2", 34);
    int v_i = 0;
    v_i = impl->getRegisterControl()->getInt32Reg("custom2", v_i);
    std::cout << std::endl
              << std::endl
              << "->Register_Int32_Custom:" << v_i << std::endl;

    impl->getRegisterControl()->setFloatReg("custom3", 0.1);
    float v_f = 0;
    v_f = impl->getRegisterControl()->getFloatReg("custom3", v_f);
    std::cout << std::endl
              << std::endl
              << "->Register_Float_Custom:" << v_f << std::endl;

    impl->getRegisterControl()->setDoubleReg("custom4", 0.2);
    double v_d = 0;
    v_d = impl->getRegisterControl()->getDoubleReg("custom4", v_d);
    std::cout << std::endl
              << std::endl
              << "->Register_Double_Custom:" << v_d << std::endl;

    impl->getRegisterControl()->setStringReg("custom5", "test");
    std::string v_s = "";
    v_s = impl->getRegisterControl()->getStringReg("custom5", v_s);
    std::cout << std::endl
              << std::endl
              << "->Register_String_Custom:" << v_s << std::endl;

    std::cout << std::endl
              << std::endl
              << "->Register_Clear_Custom:"
              << impl->getRegisterControl()->clearReg("custom5") << std::endl;
}

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(ip_local, 30004);
    rpc_cli->login("aubo", "123456");

    exampleRegisterInput(rpc_cli);
    exampleRegisterOutput(rpc_cli);
    exampleRegisterCustom(rpc_cli);
}
