import time
import numpy as np
import pyaubo_sdk

#help(pyaubo_sdk)查看各接口使用方法
help(pyaubo_sdk)

robot_ip = "192.168.199.128"
robot_port = 30000
M_PI = 3.14159265358979323846
robot_rcp_client = pyaubo_sdk.RpcClient()
robot_rtde_client = pyaubo_sdk.RtdeClient()
robot_script_client = pyaubo_sdk.ScriptClient()

def waitarrived(robot_name, target_q):
    target_q_np = np.array([target_q])
    while 1:
        current_q_np = np.array(robot_rcp_client.getRobotInterface(robot_name)
                                .getRobotState().getJointPositions())
        distance = np.linalg.norm(current_q_np - target_q_np, ord=np.Inf, axis=1)  # inf norm
        if distance < 0.0001:
            break
        time.sleep(1)

def example_movej(robot_name):
    q1 = [-2.05177, -0.400292, 1.19625, 0.0285152, 1.57033, -2.28774]
    q2 = [-0.236948, -0.465633, 1.1002, -0.00600456, 1.56824, -0.472864]
    q3 = [1.38167, -0.0115091, 1.66958, 0.107726, 1.572, 1.14576]
    robot_rcp_client.getRuntimeMachine().start()
    print("Moving to q1...")
    robot_rcp_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q1)
    print("Moving to q2...")
    robot_rcp_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q2, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q2)
    print("Moving to q3...")
    robot_rcp_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q3, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q2)

def subscribe_callback(parser):
    actual_q_ = parser.popVectorDouble()
    print('@actual_q_: {}'.format(actual_q_))
    actual_current_ = parser.popVectorDouble()
    print('@actual_current_: {}'.format(actual_current_))
    robot_mode_ = parser.popRobotModeType()
    print('@robot_mode_: {}'.format(robot_mode_))
    safety_mode_ = parser.popSafetyModeType()
    print('@safety_mode_: {}'.format(safety_mode_))
    runtime_state_ = parser.popRuntimeState()
    print('@runtime_state_: {}'.format(runtime_state_))
    line_ = parser.popInt()
    print('@line_: {}'.format(line_))
    actual_TCP_pose_ = parser.popVectorDouble()
    print('@actual_TCP_pose_: {}'.format(actual_TCP_pose_))

def example_subscribe():
    names_list = ["R1_actual_q", "R1_actual_current", "R1_robot_mode", "R1_safety_mode",
          "runtime_state", "line_number", "R1_actual_TCP_pose"]
    topic = robot_rtde_client.setTopic(False, names_list, 50, 0)
    robot_rtde_client.subscribe(topic, subscribe_callback)
    isstop=input("Input 'stop' to end ")
    if isstop=='stop':
        robot_rtde_client.removeTopic(False,topic)

def rcpClientTest():
    robot_rcp_client.connect(robot_ip, robot_port + 4)
    if robot_rcp_client.hasConnected():
        print("Robot rcp_client connected successfully!")
        robot_rcp_client.login("aubo", "123456")
        if robot_rcp_client.hasLogined():
            print("Robot rcp_client logined successfully!")
            robot_name = robot_rcp_client.getRobotNames()[0]
            if 0 == robot_rcp_client.getRobotInterface(robot_name).getRobotManage().poweron():
                print("The robot is requesting power-on!")
                if 0 == robot_rcp_client.getRobotInterface(robot_name).getRobotManage().startup():
                    print("The robot is requesting startup!")
                    while 1:
                        robot_mode = robot_rcp_client.getRobotInterface(robot_name) \
                            .getRobotState().getRobotModeType()
                        print("Robot current mode: %s" % (robot_mode.name))
                        if robot_mode == pyaubo_sdk.RobotModeType.Running:
                            break
                        time.sleep(1)
                    # write our examples here
                    example_movej(robot_name)
            if 0 == robot_rcp_client.getRobotInterface(robot_name).getRobotManage().poweroff():
                print("The robot is requesting power-off!")
    robot_rcp_client.logout()
    if ~robot_rcp_client.hasLogined():
        print("Robot rcp_client logout!")
    robot_rcp_client.disconnect()
    if ~robot_rcp_client.hasConnected():
        print("Robot rcp_client disconnected!")

def rtedClientTest():
    robot_rtde_client.connect(robot_ip, robot_port + 10)
    if robot_rtde_client.hasConnected():
        print("Robot rtde_client connected successfully!")
        robot_rtde_client.login("aubo", "123456")
        if robot_rtde_client.hasLogined():
            print("Robot rtde_client logined successfully!")
            # write our examples here
            example_subscribe()
    # robot_rtde_client.logout()
    # if ~robot_rtde_client.hasLogined():
    #     print("Robot rtde_client logout!")
    # robot_rtde_client.disconnect()
    # if ~robot_rtde_client.hasConnected():
    #     print("Robot rtde_client disconnected!")

def scriptClientTest():
    robot_script_client.connect(robot_ip, robot_port + 2)
    if robot_script_client.hasConnected():
        print("Robot script_client connected successfully!")
        robot_script_client.login("aubo", "123456")
        if robot_script_client.hasLogined():
            print("Robot script_client logined successfully!")
            # write our examples here

    robot_script_client.logout()
    if ~robot_script_client.hasLogined():
        print("Robot script_client logout!")
    robot_script_client.disconnect()
    if ~robot_script_client.hasConnected():
        print("Robot script_client disconnected!")


if __name__ == '__main__':
    rcpClientTest()
    rtedClientTest()
    scriptClientTest()