#ifndef AUBO_SDK_REGISTER_CONTROL_INTERFACE_H
#define AUBO_SDK_REGISTER_CONTROL_INTERFACE_H

#include <stdint.h>
#include <memory>
#include <aubo/global_config.h>

namespace arcs {
namespace common_interface {

/**
 * 通用寄存器
 *
 * 64 DI
 * 64 DO
 *
 * 64 INT32
 *
 * 64 FLOAT64
 */
class ARCS_ABI_EXPORT RegisterControl
{
public:
    RegisterControl();
    virtual ~RegisterControl();

    /// 输入
    bool getBoolInput(uint32_t address);
    int setBoolInput(uint32_t address, bool value);

    int getInt32Input(uint32_t address);
    int setInt32Input(uint32_t address, int value);

    float getFloatInput(uint32_t address);
    int setFloatInput(uint32_t address, float value);

    double getDoubleInput(uint32_t address);
    int setDoubleInput(uint32_t address, double value);

    /// 输出
    bool getBoolOutput(uint32_t address);
    int setBoolOutput(uint32_t address, bool value);

    int getInt32Output(uint32_t address);
    int setInt32Output(uint32_t address, int value);

    float getFloatOutput(uint32_t address);
    int setFloatOutput(uint32_t address, float value);

    double getDoubleOutput(uint32_t address);
    int setDoubleOutput(uint32_t address, double value);

    /// 通用输入输出寄存器
    bool getBoolReg(const std::string &key, bool default_value);
    int setBoolReg(const std::string &key, bool value);

    int getInt32Reg(const std::string &key, int default_value);
    int setInt32Reg(const std::string &key, int value);

    float getFloatReg(const std::string &key, float default_value);
    int setFloatReg(const std::string &key, float value);

    double getDoubleReg(const std::string &key, double default_value);
    int setDoubleReg(const std::string &key, double value);

    std::string getStringReg(const std::string &key,
                             const std::string &default_value);
    int setStringReg(const std::string &key, const std::string &value);

    int clearReg(const std::string &key);

protected:
    void *d_;
};
using RegisterControlPtr = std::shared_ptr<RegisterControl>;

// clang-format off
#define REGISTER_CONTROL_DECLARES                          \
    X(RegisterControl, 1, getBoolInput, address)           \
    X(RegisterControl, 2, setBoolInput, address, value)    \
    X(RegisterControl, 1, getInt32Input, address)          \
    X(RegisterControl, 2, setInt32Input, address, value)   \
    X(RegisterControl, 1, getFloatInput, address)          \
    X(RegisterControl, 2, setFloatInput, address, value)   \
    X(RegisterControl, 1, getDoubleInput, address)         \
    X(RegisterControl, 2, setDoubleInput, address, value)  \
    X(RegisterControl, 1, getBoolOutput, address)          \
    X(RegisterControl, 2, setBoolOutput, address, value)   \
    X(RegisterControl, 1, getInt32Output, address)         \
    X(RegisterControl, 2, setInt32Output, address, value)  \
    X(RegisterControl, 1, getFloatOutput, address)         \
    X(RegisterControl, 2, setFloatOutput, address, value)  \
    X(RegisterControl, 1, getDoubleOutput, address)        \
    X(RegisterControl, 2, setDoubleOutput, address, value) \
    X(RegisterControl, 2, getBoolReg, key, default_value)  \
    X(RegisterControl, 2, setBoolReg, key, value)          \
    X(RegisterControl, 2, getInt32Reg, key, default_value) \
    X(RegisterControl, 2, setInt32Reg, key, value)         \
    X(RegisterControl, 2, getFloatReg, key, default_value) \
    X(RegisterControl, 2, setFloatReg, key, value)         \
    X(RegisterControl, 2, getDoubleReg, key, default_value) \
    X(RegisterControl, 2, setDoubleReg, key, value)        \
    X(RegisterControl, 2, getStringReg, key, default_value) \
    X(RegisterControl, 2, setStringReg, key, value)        \
    X(RegisterControl, 1, clearReg, key)
// clang-format on
} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_REGISTER_CONTROL_INTERFACE_H
