#ifndef AUBO_SDK_ROBOT_INTERFACE_H
#define AUBO_SDK_ROBOT_INTERFACE_H

#include <aubo/sync_move.h>
#include <aubo/trace.h>
#include <aubo/robot/motion_control.h>
#include <aubo/robot/force_control.h>
#include <aubo/robot/io_control.h>
#include <aubo/robot/robot_algorithm.h>
#include <aubo/robot/robot_state.h>
#include <aubo/robot/robot_manage.h>
#include <aubo/robot/robot_config.h>
#include <aubo/global_config.h>

namespace arcs {
namespace common_interface {

/**
 * 机器人API接口
 */
class ARCS_ABI_EXPORT RobotInterface
{
public:
    RobotInterface();
    virtual ~RobotInterface();

    /// 获取RobotConfig接口
    RobotConfigPtr getRobotConfig();

    /// 获取运动规划接口
    MotionControlPtr getMotionControl();

    /// 获取力控接口
    ForceControlPtr getForceControl();

    /// 获取IO控制的接口
    IoControlPtr getIoControl();

    /// 获取同步运动接口
    SyncMovePtr getSyncMove();

    /// 获取机器人实用算法接口
    RobotAlgorithmPtr getRobotAlgorithm();

    /// 获取机器人管理接口(上电、启动、停止等)
    RobotManagePtr getRobotManage();

    /// 获取机器人状态接口
    RobotStatePtr getRobotState();

    /// 获取告警信息接口
    TracePtr getTrace();

protected:
    void *d_;
};
using RobotInterfacePtr = std::shared_ptr<RobotInterface>;

// clang-format off
#define ROBOT_INTERFACE_DECLARES              \
    X(RobotInterface, 0, getRobotConfig)      \
    X(RobotInterface, 0, getMotionControl)    \
    X(RobotInterface, 0, getForceControl)     \
    X(RobotInterface, 0, getIoControl)        \
    X(RobotInterface, 0, getSyncMove)         \
    X(RobotInterface, 0, getRobotAlgorithm)   \
    X(RobotInterface, 0, getRobotManage)      \
    X(RobotInterface, 0, getRobotState)       \
    X(RobotInterface, 0, getTrace)
// clang-format on

} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_ROBOT_INTERFACE_H
