#ifndef AUBO_SDK_TRACE_INTERFACE_H
#define AUBO_SDK_TRACE_INTERFACE_H

#include <string>
#include <vector>
#include <memory>
#include <sstream>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

/**
 * 提供给控制器扩展程序的日志记录系统
 */
class ARCS_ABI_EXPORT Trace
{
public:
    Trace();
    virtual ~Trace();

    /**
     * 向 aubo_control 日志注入告警信息
     *
     * TraceLevel:
     *  0 - FATAL
     *  1 - ERROR
     *  2 - WARNING
     *  3 - INFO
     *  4 - DEBUG
     *
     * code定义参考 error_stack
     */
    int alarm(TraceLevel level, int code,
              const std::vector<std::string> &args = {});

    /**
     * 向连接的 RTDE 客户端发送弹窗请求
     *
     * @param level
     * @param title
     * @param msg
     * @return
     */
    int popup(TraceLevel level, const std::string &title,
              const std::string &msg);

    /**
     * 弹出输入框
     *
     * @param title
     * @param type
     *         1: bool
     *         2: int
     *         3: double
     *         4: string
     * @return
     */
    int popupInputBox(const std::string &title, int type);

    /**
     * peek最新的 AlarmInfo(上次一获取之后)
     * last_time设置为0时，可以获取到所有的AlarmInfo
     */
    RobotMsgVector peek(size_t num, uint64_t last_time = 0);

protected:
    void *d_;
};

using TracePtr = std::shared_ptr<Trace>;

// clang-format off
#define TRACE_DECLARES                     \
    X(Trace, 3, alarm, level, code, args)  \
    X(Trace, 3, popup, level, title, msg)  \
    X(Trace, 2, popupInputBox, title, type)  \
    X(Trace, 2, peek, num, last_time)
// clang-format on
} // namespace common_interface
} // namespace arcs
#endif
