#ifndef AUBO_SDK_SCRIPT_H
#define AUBO_SDK_SCRIPT_H

#include <string>
#include <memory>
#include <functional>
#include <aubo/global_config.h>

namespace arcs {
namespace aubo_sdk {

class ScriptWriter
{
public:
    virtual ~ScriptWriter() = default;

    virtual ScriptWriter &append(const std::string &line) = 0;
    virtual ScriptWriter &append(const char *buf, size_t len) = 0;
    virtual ScriptWriter &moveJoint() = 0;
    virtual ScriptWriter &moveLine() = 0;
    virtual ScriptWriter &ifCondition() = 0;
    virtual ScriptWriter &elseCondition() = 0;
    virtual ScriptWriter &elseIfCondition() = 0;
    virtual ScriptWriter &whileCondition() = 0;
    virtual ScriptWriter &end() = 0;
};

class ARCS_ABI_EXPORT ScriptClient
{
public:
    ScriptClient();
    ~ScriptClient();

    // 0: LOGLEVEL_FATAL
    // 1: LOGLEVEL_ERROR
    // 2: LOGLEVEL_WARNING
    // 3: LOGLEVEL_INFO
    // 4: LOGLEVEL_DEBUG
    // 5: LOGLEVEL_BACKTRACE
    void setLogHandler(
        std::function<void(int /*level*/, const char * /*filename*/,
                           int /*line*/, const std::string & /*message*/)>
            handler);

    /// 连接到服务器
    int connect(const std::string &ip, int port);

    /// 是否处于连接状态
    bool hasConnected() const;

    /// 登录
    int login(const std::string &usrname, const std::string &passwd);

    /// 返回客户端是否登录
    bool hasLogined();

    /// 登出
    int logout();

    /// 断开连接
    int disconnect();

    /// 发送文件
    int sendFile(const std::string &path);

    /// 发送文件
    int sendString(const std::string &script);

    /// 使用ScriptWriter构建脚本
    int send(const std::string &chunck_name,
             std::function<int(ScriptWriter &)> cb);

    int subscribeVariableUpdate(
        std::function<void(const std::string &, const std::string &)> cb);

    int subscribeScriptError(std::function<void(const std::string &)> cb);

private:
    class Impl;
    Impl *impl;
};
using ScriptClientPtr = std::shared_ptr<ScriptClient>;

} // namespace aubo_sdk
} // namespace arcs
#endif
